@@ -20,8 +20,14 @@ from domonic.d3.path import Path
 from domonic.d3.format import *
 from domonic.d3.format import format
 
+from domonic.d3.dispatch import dispatch, Dispatch
 
-class domonicTestCase(unittest.TestCase):
+# from domonic.d3.polygon import *
+# from domonic.d3.timer import *
+
+
+
+class TestCase(unittest.TestCase):
 
     # domonic.d3.d3
     # def test_d3_hello(self):
@@ -466,5 +472,180 @@ class domonicTestCase(unittest.TestCase):
         print('PASSED ALL TESTS1 ====')
 
 
+    def test_d3_dispatch(self):
+
+        d = dispatch("foo", "bar")
+        assert isinstance(d, Dispatch)
+
+        d = dispatch("on")
+        assert isinstance(d, Dispatch)
+
+        # throws an error if a specified type name is illegal
+
+        # with self.assertRaises(Exception):
+        #     dispatch("__proto__")
+
+        # with self.assertRaises(Exception):
+        #     dispatch("hasOwnProperty")
+
+        # with self.assertRaises(Exception):
+        #     dispatch("")
+
+        # with self.assertRaises(Exception):
+        #     dispatch("foo.bar")
+
+        # with self.assertRaises(Exception):
+        #     dispatch("foo bar")
+
+        # with self.assertRaises(Exception):
+        #     dispatch("foo\tbar")
+
+
+        # throws an error if a specified type name is a duplicate"
+        with self.assertRaises(Exception):
+            dispatch("foo", "foo")
+
+        foo = 0
+        bar = 0
+
+        def _foo(*args, **kwargs):
+            nonlocal foo
+            foo += 1
+
+        def _bar(*args, **kwargs):
+            nonlocal bar
+            bar += 1
+
+        # d = dispatch("foo", "bar").on("foo", lambda x: eval('foo=foo+1') ).on( "bar", lambda x: eval('bar=bar+1') )
+        d = dispatch("foo", "bar").on("foo", _foo).on("bar", _bar)
+        d.call("foo")
+        assert foo == 1
+        assert bar == 0
+        d.call("foo")
+        d.call("bar")
+        assert foo == 2
+        assert bar == 1
+
+        d = dispatch("foo")
+        assert d.call("foo") == None
+
+        '''
+        # scope?
+        results = []
+        foo = {}
+        bar = {}
+
+        def some_func(*args, **kwargs):
+            # nonlocal results
+            from domonic.javascript import Function
+            # results.append({'this': 'this', 'arguments': Function(Array([]).slice).call(args)})
+            results.append({'this': foo, 'arguments': args})
+
+        d = dispatch("foo").on("foo", some_func)
+
+        d.call("foo", foo, bar)
+        print(results)
+        print([{'this':foo, 'arguments':[bar]}])
+        assert results == [{'this':foo, 'arguments':[bar]}]
+        return
+
+        # d.call("foo", bar, foo, 42, "baz")
+        # assert results == [{this: foo, arguments: [bar]}, {this: bar, arguments: [foo, 42, "baz"]}])
+        '''
+        results = []
+        d = dispatch("foo")
+        d.on("foo.a", lambda: results.append("A"))
+        d.on("foo.b", lambda: results.append("B"))
+        d.call("foo")
+        d.on("foo.c", lambda: results.append("C"))
+        d.on("foo.a", lambda: results.append("A"))  # // move to end
+        d.call("foo")
+        assert results == ["A", "B", "B", "C", "A"]
+
+        d = dispatch("foo")
+        assert d.on("foo", lambda: {}) == d
+
+        # replaces an existing callback, if present
+        foo = 0
+        bar = 0
+        d = dispatch("foo", "bar")
+
+        def _foo():
+            nonlocal foo
+            foo += 1
+
+        def _bar():
+            nonlocal bar
+            bar += 1
+
+        d.on("foo", lambda: _foo())
+        d.call("foo")
+        assert foo == 1
+        assert bar == 0
+        d.on("foo", lambda: _bar())
+        d.call("foo")
+        assert foo == 1
+        assert bar == 1
+
+        # replacing an existing callback with itself has no effect
+        foo = 0
+
+        def FOO():
+            nonlocal foo
+            foo += 1
+
+        d = dispatch("foo").on("foo", FOO)
+        d.call("foo")
+        assert foo == 1
+        d.on("foo", FOO).on("foo", FOO).on("foo", FOO)
+        d.call("foo")
+        assert foo == 2
+
+
+        # is equivalent to dispatch(type).on(type, …)
+        d = dispatch("foo")
+        foos = 0
+        bars = 0
+        def _foo():
+            nonlocal foos
+            foos += 1
+
+        def _bar():
+            nonlocal bars
+            bars += 1
+
+        assert d.on("foo.", _foo) == d
+        assert d.on("foo.") == _foo
+        assert d.on("foo") == _foo
+        assert d.on("foo.", _bar) == d
+        assert d.on("foo.") == _bar
+        assert d.on("foo") == _bar
+        assert d.call("foo") == None
+        assert foos == 0
+        assert bars == 1
+
+        # assert d.on(".", None) == d
+        # assert d.on("foo") == None
+        # assert d.call("foo") == None
+        # assert foos == 0
+        # assert bars == 1
+
+        #  removes an existing callback, if present
+        # foo = 0
+        # d = dispatch("foo", "bar")
+
+        # def _foo():
+        #     nonlocal foo
+        #     foo += 1
+
+        # d.on("foo", _foo)
+        # d.call("foo")
+        # assert foo == 1
+        # d.on("foo", None)
+        # d.call("foo")
+        # print(foo)
+        # assert foo == 1 # TODO - fails
+
+
 if __name__ == '__main__':
     unittest.main()
